"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var DEFAULT_MAX = 5;
/**
 * Raw throttle function, which can return extra meta data.
 * @param tasks array[] of functions
 * @param maxInProgress integer with the max amount of tasks to be run in parallel
 * @param failFast boolean if true, do fail-fast behaviour (see Promise.all() documentation)
 * @param progressCallback function which can be used to see the progress
 * @param nextCheck
 * @returns {Promise}
 */
function raw(tasks, maxInProgress, failFast, progressCallback, nextCheck) {
    if (maxInProgress === void 0) { maxInProgress = DEFAULT_MAX; }
    if (failFast === void 0) { failFast = false; }
    if (nextCheck === void 0) { nextCheck = defaultNextTaskCheck; }
    return new Promise(function (resolve, reject) {
        var result = {
            amountDone: 0,
            amountStarted: 0,
            amountResolved: 0,
            amountRejected: 0,
            rejectedIndexes: [],
            resolvedIndexes: [],
            taskResults: []
        };
        if (tasks.length === 0) {
            return resolve(result);
        }
        var failedFast = false;
        var amountQueued = 0;
        var executeTask = function (index) {
            if (typeof tasks[index] === 'function') {
                tasks[index]()
                    .then(function (taskResult) {
                    result.taskResults[index] = taskResult;
                    result.resolvedIndexes.push(index);
                    result.amountResolved++;
                    taskDone();
                }, function (error) {
                    result.taskResults[index] = error;
                    result.rejectedIndexes.push(index);
                    result.amountRejected++;
                    if (failFast) {
                        failedFast = true;
                        return reject(result);
                    }
                    taskDone();
                });
            }
            else {
                failedFast = true;
                return reject(new Error('tasks[' + index + ']: ' + tasks[index] + ', is supposed to be of type function'));
            }
        };
        var taskDone = function () {
            //make sure no more tasks are spawned when we failedFast
            if (failedFast) {
                return;
            }
            result.amountDone++;
            if (progressCallback) {
                progressCallback(result);
            }
            if (result.amountDone === tasks.length) {
                return resolve(result);
            }
            if (amountQueued < tasks.length) {
                amountQueued++;
                nextTask();
            }
        };
        var nextTask = function () {
            //check if we can execute the next task
            nextCheck(result, tasks)
                .then(function (canExecuteNextTask) {
                if (canExecuteNextTask === true) {
                    //execute it
                    executeTask(result.amountStarted++);
                }
            }, reject);
        };
        //spawn the first X task
        for (var i = 0; i < Math.min(maxInProgress, tasks.length); i++) {
            amountQueued++;
            nextTask();
        }
    });
}
exports.raw = raw;
/**
 * Default checker which validates if a next task should begin.
 * This can be overwritten to write own checks for example checking the amount
 * of used ram and waiting till the ram is low enough for a next task.
 *
 * It should always resolve with a boolean, either `true` to start a next task
 * or `false` to stop executing a new task.
 *
 * If this method rejects, the error will propagate to the caller
 * @param status
 * @param tasks
 * @returns {Promise}
 */
var defaultNextTaskCheck = function (status, tasks) {
    return new Promise(function (resolve, reject) {
        resolve(status.amountStarted < tasks.length);
    });
};
/**
 * Simply run all the promises after each other, so in synchronous manner
 * @param tasks required array of tasks to be executed
 * @param failFast optional boolean if we directly reject on a single error defaults to true
 * @param progressCallback optional function to be run to get status updates
 * @param nextCheck function which should return a promise, when resolved the next task will spawn
 */
function sync(tasks, failFast, progressCallback, nextCheck) {
    if (failFast === void 0) { failFast = true; }
    if (nextCheck === void 0) { nextCheck = defaultNextTaskCheck; }
    return new Promise(function (resolve, reject) {
        raw(tasks, 1, failFast, progressCallback, nextCheck)
            .then(function (result) {
            resolve(result.taskResults);
        }, function (error) {
            if (error instanceof Error) {
                reject(error);
            }
            else {
                reject(error.taskResults[error.rejectedIndexes[0]]);
            }
        });
    });
}
exports.sync = sync;
/**
 * Exposes the same behaviour as Promise.All(), but throttled!
 * @param tasks required array of tasks to be executed
 * @param maxInProgress optional integer max amount of parallel tasks to be run defaults to 5
 * @param failFast optional boolean if we directly reject on a single error defaults to true
 * @param progressCallback optional function to be run to get status updates
 * @param nextCheck function which should return a promise, when resolved the next task will spawn
 */
function all(tasks, maxInProgress, failFast, progressCallback, nextCheck) {
    if (maxInProgress === void 0) { maxInProgress = DEFAULT_MAX; }
    if (failFast === void 0) { failFast = true; }
    if (nextCheck === void 0) { nextCheck = defaultNextTaskCheck; }
    return new Promise(function (resolve, reject) {
        raw(tasks, maxInProgress, failFast, progressCallback, nextCheck)
            .then(function (result) {
            resolve(result.taskResults);
        }, function (error) {
            if (error instanceof Error) {
                reject(error);
            }
            else {
                reject(error.taskResults[error.rejectedIndexes[0]]);
            }
        });
    });
}
exports.all = all;
//# sourceMappingURL=throttle.js.map